@@ -45,6 +45,16 @@
 #include "sdpd.h"
 #include "log.h"
 
+typedef struct {
+	uint32_t timestamp;
+	union {
+		uint16_t maxBytesSent;
+		uint16_t lastIndexSent;
+	} cStateValue;
+} sdp_cont_state_t;
+
+#define SDP_CONT_STATE_SIZE (sizeof(uint8_t) + sizeof(sdp_cont_state_t))
+
 #define MIN(x, y) ((x) < (y)) ? (x): (y)
 
 typedef struct _sdp_cstate_list sdp_cstate_list_t;
@@ -58,7 +68,7 @@ struct _sdp_cstate_list {
 static sdp_cstate_list_t *cstates;
 
 // FIXME: should probably remove it when it's found
-sdp_buf_t *sdp_get_cached_rsp(sdp_cont_state_t *cstate)
+static sdp_buf_t *sdp_get_cached_rsp(sdp_cont_state_t *cstate)
 {
 	sdp_cstate_list_t *p;
 
@@ -417,7 +427,7 @@ static int service_search_req(sdp_req_t *req, sdp_buf_t *buf)
 
 		handleSize = 0;
 		for (; list && rsp_count < expected; list = list->next) {
-			sdp_record_t *rec = (sdp_record_t *) list->data;
+			sdp_record_t *rec = list->data;
 
 			SDPDBG("Checking svcRec : 0x%x", rec->handle);
 
@@ -547,17 +557,13 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, sdp_buf_t *buf)
 	if (!rec)
 		return SDP_INVALID_RECORD_HANDLE;
 
-	if (seq) {
-		SDPDBG("Entries in attr seq : %d", sdp_list_len(seq));
-	} else {
-		SDPDBG("NULL attribute descriptor");
-	}
-
 	if (seq == NULL) {
 		SDPDBG("Attribute sequence is NULL");
 		return 0;
 	}
 
+	SDPDBG("Entries in attr seq : %d", sdp_list_len(seq));
+
 	sdp_gen_record_pdu(rec, &pdu);
 
 	for (; seq; seq = seq->next) {
@@ -567,7 +573,7 @@ static int extract_attrs(sdp_record_t *rec, sdp_list_t *seq, sdp_buf_t *buf)
 
 		if (aid->dtd == SDP_UINT16) {
 			uint16_t attr = bt_get_unaligned((uint16_t *)&aid->uint16);
-			sdp_data_t *a = (sdp_data_t *)sdp_data_get(rec, attr);
+			sdp_data_t *a = sdp_data_get(rec, attr);
 			if (a)
 				sdp_append_to_pdu(buf, a);
 		} else if (aid->dtd == SDP_UINT32) {
@@ -667,7 +673,7 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	mlen = scanned + sizeof(uint32_t) + sizeof(uint16_t) + 1;
 	// ensure we don't read past buffer
 	if (plen < mlen || plen != mlen + *(uint8_t *)pdata) {
-		status = SDP_INVALID_SYNTAX;
+		status = SDP_INVALID_PDU_SIZE;
 		goto done;
 	}
 
@@ -683,6 +689,15 @@ static int service_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 	SDPDBG("SvcRecHandle : 0x%x", handle);
 	SDPDBG("max_rsp_size : %d", max_rsp_size);
 
+	/*
+	 * Check that max_rsp_size is within valid range
+	 * a minimum size of 0x0007 has to be used for data field
+	 */
+	if (max_rsp_size < 0x0007) {
+		status = SDP_INVALID_SYNTAX;
+		goto done;
+	}
+
 	/*
 	 * Calculate Attribute size acording to MTU
 	 * We can send only (MTU - sizeof(sdp_pdu_hdr_t) - sizeof(sdp_cont_state_t))
@@ -818,7 +833,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 
 	plen = ntohs(((sdp_pdu_hdr_t *)(req->buf))->plen);
 	if (plen < totscanned || plen != totscanned + *(uint8_t *)pdata) {
-		status = SDP_INVALID_SYNTAX;
+		status = SDP_INVALID_PDU_SIZE;
 		goto done;
 	}
 
@@ -852,7 +867,7 @@ static int service_search_attr_req(sdp_req_t *req, sdp_buf_t *buf)
 		/* no continuation state -> create new response */
 		sdp_list_t *p;
 		for (p = svcList; p; p = p->next) {
-			sdp_record_t *rec = (sdp_record_t *) p->data;
+			sdp_record_t *rec = p->data;
 			if (sdp_match_uuid(pattern, rec->pattern) > 0 &&
 					sdp_check_access(rec->handle, &req->device)) {
 				rsp_count++;
